from vectara_client.core import Factory
from vectara_client.models import CreateCorpusRequest, StructuredDocument, CreateDocumentRequest
from vectara_client.documents import DocumentManager
from vectara_client.corpus import CorpusManager, CorpusBuilder
from time import sleep
from pathlib import Path
import logging
import unittest

class DocumentManagerTest(unittest.TestCase):

    CORPUS_NAME = "Test-DocumentManager"
    CORPUS_KEY = "test-doc-manager"
    DOCUMENT_ID = "HR Complaint Process"
    TEST_DOCUMENT = "resources/hr_complaints/complaints.json"

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        logging.basicConfig(format='%(asctime)s:%(name)-35s %(levelname)s:%(message)s', level=logging.INFO,
                            datefmt='%H:%M:%S %z')
        self.logger = logging.getLogger(self.__class__.__name__)

    def test_check_document_exists(self):
        client = Factory().build()
        corpus_manager = client.corpus_manager

        self.logger.info("Checking if we need to setup our corpus")

        corpus = corpus_manager.find_corpus_by_name(self.CORPUS_NAME, fail_if_not_exist=False)
        if corpus:
            corpus_key = corpus.key
            self.logger.info(f"Existing corpus found with key [{corpus_key}]")
        else:
            self.logger.info(f"Creating new corpus with key [{self.CORPUS_KEY}]")

            corpus_manager.builder()

            request: CreateCorpusRequest = CreateCorpusRequest.from_dict({
                "name": self.CORPUS_NAME,
                "key": self.CORPUS_KEY
            })
            corpus = corpus_manager.create_corpus(request)
            corpus_key = corpus.key
            self.logger.info(f"New corpus created with id [{corpus.id}]")

        # Delete All in Corpus
        docs_response = client.documents_api.list_corpus_documents(corpus_key)
        if docs_response and docs_response.documents:
            for doc in docs_response.documents:
                self.logger.info(f"Deleting document [{doc.id}]")
                client.documents_api.delete_corpus_document(corpus_key, doc.id)

        # Sleep to ensure it's been deleted.
        sleep(5)

        # First run the negative.
        document_manager = client.document_manager
        self.assertFalse(document_manager.check_document_exists(corpus_key, self.DOCUMENT_ID))

        path = Path(self.TEST_DOCUMENT)
        result = document_manager.add_document(corpus_key, doc_path=path)

        self.logger.info(f"Waiting 5 seconds after upload")

        # Sleep to give time for eventual consistency to ingest document.
        sleep(5)

        self.assertTrue(document_manager.check_document_exists(corpus_key, self.DOCUMENT_ID))

        self.logger.info(f"Deleting test document [{self.DOCUMENT_ID}], waiting 5 seconds")
        client.documents_api.delete_corpus_document(corpus_key, self.DOCUMENT_ID)
        # Sleep to ensure it's been deleted.
        sleep(5)

        self.assertFalse(document_manager.check_document_exists(corpus_key, self.DOCUMENT_ID))












if __name__ == '__main__':
    unittest.main()



from vectara_client.core import Factory
from vectara_client.models import CreateCorpusRequest, StructuredDocument, CreateDocumentRequest
from vectara_client.documents import DocumentManager
from vectara_client.corpus import CorpusManager, CorpusBuilder
from time import sleep
from pathlib import Path
import logging
import unittest

class CorpusBuilderTest(unittest.TestCase):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        logging.basicConfig(format='%(asctime)s:%(name)-35s %(levelname)s:%(message)s', level=logging.INFO,
                            datefmt='%H:%M:%S %z')
        self.logger = logging.getLogger(self.__class__.__name__)

    def test_corpus_key(self):
        builder = CorpusBuilder("My Corpus  Yay")
        self.assertEqual(builder.build().key, "my-corpus-yay")


    def test_invalid_key(self):
        with self.assertRaises(ValueError) as context:
            CorpusBuilder("My Corpus Yay", "Non valid KEY^&")

        self.assertTrue(r"must validate the regular expression /[a-zA-Z0-9_\=\-]+$/" in context.exception.args[0])

        with self.assertRaises(ValueError) as context:
            builder = CorpusBuilder("My Corpus Yay")
            builder.key("Non valid KEY^&")

        self.assertTrue(r"must validate the regular expression /[a-zA-Z0-9_\=\-]+$/" in context.exception.args[0])

    def test_add_attribute(self):
        corpus = (CorpusBuilder("filter-attrs")
                  .add_attribute("first", "MyFirstAttr")
                  .add_attribute("second", type="boolean", indexed=False)
                  .add_attribute("third", document=False)
                  .build()
                  )
        # Test the first filter attribute
        first = corpus.filter_attributes[0]
        self.assertEqual(first.name, "first")
        self.assertEqual(first.description, "MyFirstAttr")
        self.assertTrue(first.indexed)
        self.assertEqual(first.type, "text")
        self.assertEqual(first.level, "document")

        # Test the second filter attribute
        second = corpus.filter_attributes[1]
        self.assertEqual(second.name, "second")
        self.assertIsNone(second.description)
        self.assertEqual(second.type, "boolean")
        self.assertFalse(second.indexed)

        # Test the third filter attribute
        third = corpus.filter_attributes[2]
        self.assertEqual(third.name, "third")
        self.assertEqual(third.level, "part")

    def test_add_attribute_invalid_type(self):
        with self.assertRaises(ValueError) as context:
            CorpusBuilder("filter-attrs").add_attribute("first", type="unknown")


        self.assertTrue("must be one of enum values ('integer', 'real_number', 'text', 'boolean', 'list[integer]', 'list[real_number]', 'list[text]')" in context.exception.args[0])

    def test_add_dimension(self):
        corpus = (CorpusBuilder("dimensions")
                  .add_dimension("my-dimension",
                                 0.1, 0.2)
                  .build())

        dimension = corpus.custom_dimensions[0]
        self.assertEqual(dimension.name, "my-dimension")
        self.assertEqual(dimension.indexing_default, 0.1)
        self.assertEqual(dimension.querying_default, 0.2)








class CorpusManagerTest(unittest.TestCase):

    CORPUS_NAME = "Test-DocumentManager"
    CORPUS_KEY = "test-doc-manager"
    DOCUMENT_ID = "HR Complaint Process"
    TEST_DOCUMENT = "resources/hr_complaints/complaints.json"

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        logging.basicConfig(format='%(asctime)s:%(name)-35s %(levelname)s:%(message)s', level=logging.INFO,
                            datefmt='%H:%M:%S %z')
        self.logger = logging.getLogger(self.__class__.__name__)




# coding: utf-8

"""
    Vectara REST API v2

    Vectara provides an end-to-end platform for creating GenAI products using  a simple to use API.  You can [sign up for an account](https://console.vectara.com/signup) and  then view several [API Recipes](https://docs.vectara.com/docs/api-recipes) with example queries  and parameter values.  The Vectara API Playground lets you experiment with REST endpoints from  your browser. Select an endpoint to view its definition, including the  required or optional headers, body, responses, and sample commands. On the  right side of each endpoint page, like [Get Corpus](/docs/rest-api/get-corpus), you manually  enter your API Key or OAuth Bearer Token, `customer_id`, and then any  required body parameters like the `corpusID` before sending the API  request.  :::note  Vectara supports two primary methods of authentication: API keys and OAuth  2.0, which are applicable to all endpoints. Query API Keys are used for  read-only querying operations, while Index API Keys provide read and write  access. The OAuth 2.0 operations authenticate with a Bearer Token via the  OAuth 2.0 client credentials grant. Review the [**OAuth 2.0 section**](https://docs.vectara.com/docs/learn/authentication/oauth-2) about  how to generate the token. 

    The version of the OpenAPI document: 2.0.0
    Contact: feedback@vectara.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
import pprint
from pydantic import BaseModel, ConfigDict, Field, StrictStr, ValidationError, field_validator
from typing import Any, List, Optional
from vectara_client.models.core_document import CoreDocument
from vectara_client.models.structured_document import StructuredDocument
from pydantic import StrictStr, Field
from typing import Union, List, Set, Optional, Dict
from typing_extensions import Literal, Self

CREATEDOCUMENTREQUEST_ONE_OF_SCHEMAS = ["CoreDocument", "StructuredDocument"]

class CreateDocumentRequest(BaseModel):
    """
    Creating a document using this endpoint can take multiple forms depending on how much control of the resulting document parts you desire. You can create a document with natural structure, and Vectara will use it's proprietary strategy to create document parts.  Otherwise, you can create a document with all the document parts explicitly specified.  A document part is the a search result item in search and retrieval augmented generation endpoints. 
    """
    # data type: CoreDocument
    oneof_schema_1_validator: Optional[CoreDocument] = None
    # data type: StructuredDocument
    oneof_schema_2_validator: Optional[StructuredDocument] = None
    actual_instance: Optional[Union[CoreDocument, StructuredDocument]] = None
    one_of_schemas: Set[str] = { "CoreDocument", "StructuredDocument" }

    model_config = ConfigDict(
        validate_assignment=True,
        protected_namespaces=(),
    )


    discriminator_value_class_map: Dict[str, str] = {
    }

    def __init__(self, *args, **kwargs) -> None:
        if args:
            if len(args) > 1:
                raise ValueError("If a position argument is used, only 1 is allowed to set `actual_instance`")
            if kwargs:
                raise ValueError("If a position argument is used, keyword arguments cannot be used.")
            super().__init__(actual_instance=args[0])
        else:
            super().__init__(**kwargs)

    @field_validator('actual_instance')
    def actual_instance_must_validate_oneof(cls, v):
        instance = CreateDocumentRequest.model_construct()
        error_messages = []
        match = 0
        # validate data type: CoreDocument
        if not isinstance(v, CoreDocument):
            error_messages.append(f"Error! Input type `{type(v)}` is not `CoreDocument`")
        else:
            match += 1
        # validate data type: StructuredDocument
        if not isinstance(v, StructuredDocument):
            error_messages.append(f"Error! Input type `{type(v)}` is not `StructuredDocument`")
        else:
            match += 1
        if match > 1:
            # more than 1 match
            raise ValueError("Multiple matches found when setting `actual_instance` in CreateDocumentRequest with oneOf schemas: CoreDocument, StructuredDocument. Details: " + ", ".join(error_messages))
        elif match == 0:
            # no match
            raise ValueError("No match found when setting `actual_instance` in CreateDocumentRequest with oneOf schemas: CoreDocument, StructuredDocument. Details: " + ", ".join(error_messages))
        else:
            return v

    @classmethod
    def from_dict(cls, obj: Union[str, Dict[str, Any]]) -> Self:
        return cls.from_json(json.dumps(obj))

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Returns the object represented by the json string"""
        instance = cls.model_construct()
        error_messages = []
        match = 0

        # deserialize data into CoreDocument
        try:
            instance.actual_instance = CoreDocument.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into StructuredDocument
        try:
            instance.actual_instance = StructuredDocument.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))

        if match > 1:
            # more than 1 match
            raise ValueError("Multiple matches found when deserializing the JSON string into CreateDocumentRequest with oneOf schemas: CoreDocument, StructuredDocument. Details: " + ", ".join(error_messages))
        elif match == 0:
            # no match
            raise ValueError("No match found when deserializing the JSON string into CreateDocumentRequest with oneOf schemas: CoreDocument, StructuredDocument. Details: " + ", ".join(error_messages))
        else:
            return instance

    def to_json(self) -> str:
        """Returns the JSON representation of the actual instance"""
        if self.actual_instance is None:
            return "null"

        if hasattr(self.actual_instance, "to_json") and callable(self.actual_instance.to_json):
            return self.actual_instance.to_json()
        else:
            return json.dumps(self.actual_instance)

    def to_dict(self) -> Optional[Union[Dict[str, Any], CoreDocument, StructuredDocument]]:
        """Returns the dict representation of the actual instance"""
        if self.actual_instance is None:
            return None

        if hasattr(self.actual_instance, "to_dict") and callable(self.actual_instance.to_dict):
            return self.actual_instance.to_dict()
        else:
            # primitive type
            return self.actual_instance

    def to_str(self) -> str:
        """Returns the string representation of the actual instance"""
        return pprint.pformat(self.model_dump())





import json
import re

import urllib3

from vectara_client.exceptions import ApiException, ApiValueError
from urllib3.fields import RequestField

from vectara_client.rest import RESTClientObject, RESTResponse


class OurRESTClientObject(RESTClientObject):

    def request(
        self,
        method,
        url,
        headers=None,
        body=None,
        post_params=None,
        _request_timeout=None
    ):
        """Perform requests.

        :param method: http request method
        :param url: http request url
        :param headers: http request headers
        :param body: request json body, for `application/json`
        :param post_params: request post parameters,
                            `application/x-www-form-urlencoded`
                            and `multipart/form-data`
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        """
        method = method.upper()
        assert method in [
            'GET',
            'HEAD',
            'DELETE',
            'POST',
            'PUT',
            'PATCH',
            'OPTIONS'
        ]

        if post_params and body:
            raise ApiValueError(
                "body parameter cannot be used with post_params parameter."
            )

        post_params = post_params or {}
        headers = headers or {}

        timeout = None
        if _request_timeout:
            if isinstance(_request_timeout, (int, float)):
                timeout = urllib3.Timeout(total=_request_timeout)
            elif (
                    isinstance(_request_timeout, tuple)
                    and len(_request_timeout) == 2
                ):
                timeout = urllib3.Timeout(
                    connect=_request_timeout[0],
                    read=_request_timeout[1]
                )

        try:
            # For `POST`, `PUT`, `PATCH`, `OPTIONS`, `DELETE`
            if method in ['POST', 'PUT', 'PATCH', 'OPTIONS', 'DELETE']:

                # no content type provided or payload is json
                content_type = headers.get('Content-Type')
                if (
                    not content_type
                    or re.search('json', content_type, re.IGNORECASE)
                ):
                    request_body = None
                    if body is not None:
                        request_body = json.dumps(body)
                    r = self.pool_manager.request(
                        method,
                        url,
                        body=request_body,
                        timeout=timeout,
                        headers=headers,
                        preload_content=False
                    )
                elif content_type == 'application/x-www-form-urlencoded':
                    r = self.pool_manager.request(
                        method,
                        url,
                        fields=post_params,
                        encode_multipart=False,
                        timeout=timeout,
                        headers=headers,
                        preload_content=False
                    )
                elif content_type == 'multipart/form-data':
                    # must del headers['Content-Type'], or the correct
                    # Content-Type which generated by urllib3 will be
                    # overwritten.
                    del headers['Content-Type']
                    # Ensures that dict objects are serialized
                    post_params = [(a, json.dumps(b)) if isinstance(b, dict) else (a,b) for a, b in post_params]

                    ## Manually override metadata tuple to be application json.
                    for index, post_param in enumerate(post_params):
                        field_name = post_param[0]
                        if field_name == "metadata":

                            field = RequestField(field_name, post_param[1])
                            field.make_multipart(content_type="application/json")
                            post_params[index] = field
                            break

                    r = self.pool_manager.request(
                        method,
                        url,
                        fields=post_params,
                        encode_multipart=True,
                        timeout=timeout,
                        headers=headers,
                        preload_content=False
                    )
                # Pass a `string` parameter directly in the body to support
                # other content types than JSON when `body` argument is
                # provided in serialized form.
                elif isinstance(body, str) or isinstance(body, bytes):
                    r = self.pool_manager.request(
                        method,
                        url,
                        body=body,
                        timeout=timeout,
                        headers=headers,
                        preload_content=False
                    )
                elif headers['Content-Type'] == 'text/plain' and isinstance(body, bool):
                    request_body = "true" if body else "false"
                    r = self.pool_manager.request(
                        method,
                        url,
                        body=request_body,
                        preload_content=False,
                        timeout=timeout,
                        headers=headers)
                else:
                    # Cannot generate the request from given parameters
                    msg = """Cannot prepare a request message for provided
                             arguments. Please check that your arguments match
                             declared content type."""
                    raise ApiException(status=0, reason=msg)
            # For `GET`, `HEAD`
            else:
                r = self.pool_manager.request(
                    method,
                    url,
                    fields={},
                    timeout=timeout,
                    headers=headers,
                    preload_content=False
                )
        except urllib3.exceptions.SSLError as e:
            msg = "\n".join([type(e).__name__, str(e)])
            raise ApiException(status=0, reason=msg)

        return RESTResponse(r)


from vectara_client.api import QueriesApi, UploadApi, IndexApi, DocumentsApi
from vectara_client.models import (QueryCorpusRequest, SearchCorpusParameters, IndividualSearchResult,
                                   StructuredDocument, CoreDocument, CreateDocumentRequest)
from types import MappingProxyType
from typing import Union, Any
from pathlib import Path

import logging
import json


class DocumentManager:

    DEFAULT_METADATA: dict[str: Any] = MappingProxyType({})

    def __init__(self, queries_api: QueriesApi, upload_api: UploadApi, index_api: IndexApi,
                 documents_api: DocumentsApi):
        self.logger = logging.getLogger(self.__class__.__name__)
        self.queries_api = queries_api
        self.upload_api = upload_api
        self.index_api = index_api
        self.documents_api = documents_api

    def get_document(self, corpus_key: str, document_id: str) -> IndividualSearchResult:
        self.logger.info(f"Retrieving document [{document_id}] from corpus [{corpus_key}]")
        search = SearchCorpusParameters(
            metadata_filter=f"(doc.id = '{document_id}')"
        )

        request = QueryCorpusRequest(
            query='',
            search=search,
            generation=None
        )

        response = self.queries_api.query_corpus(corpus_key=corpus_key, query_corpus_request=request)
        if response.search_results and len(response.search_results) > 0:
            self.logger.info(f"Returning document [{document_id}] from corpus [{corpus_key}]")
            return response.search_results[0]
        else:
            self.logger.info(f"Did not find document [{document_id}] in corpus [{corpus_key}]")

    def check_document_exists(self, corpus_key: str, document_id: str):
        self.logger.info(f"Checking if document [{document_id}] exists in corpus [{corpus_key}]")
        if self.get_document(corpus_key, document_id):
            self.logger.info(f"Document [{document_id}] exists in corpus [{corpus_key}]")
            return True
        else:
            self.logger.info(f"Document [{document_id}] does not exist in corpus [{corpus_key}]")
            return False

    def add_document(self, corpus_key: str, doc_path: Path = None, doc_json: str = None,
                     content: Union[StructuredDocument, CoreDocument, dict[str, Any]] = None,
                     encoding: str = "utf-8", metadata: dict[str, Any] = None,
                     check_exists:bool = True, overwrite: bool = True) -> bool:
        # First validate inputs.
        if doc_path:
            file_name = str(doc_path.resolve())

            if not doc_path.name.lower().endswith("json"):
                # Process as upload.
                doc_id = file_name

                if self.check_document_exists(corpus_key, doc_id):
                    self.logger.info("The document with id [" + doc_id + "] already exists")
                    if overwrite:
                        self.logger.info("Overwriting document, deleting old version")
                        self.documents_api.delete_corpus_document(corpus_key, doc_id)
                    else:
                        self.logger.info("Skipping this document as overwrite set to false")
                        return False


                self.logger.info(f"We will ingest [{file_name}] as a binary type")

                if metadata:
                    self.upload_api.upload_file(corpus_key, file_name, metadata=metadata)
                else:
                    self.upload_api.upload_file(corpus_key, file_name, metadata)
                return True
            else:
                self.logger.info(f"Loading document to index from path [{file_name}]")
                with open(doc_path, "r", encoding=encoding) as r:
                    doc_json = r.read()

        if doc_json:
            content = json.loads(doc_json)

        parsed_content: Union[StructuredDocument, CoreDocument]

        if content:
            if isinstance(content, dict):
                self.logger.debug("Parsing content from dict")
                doc_type = content["type"]

                if not doc_type:
                    raise Exception("For API v2, you must provide a type attribute (core or structured).")
                if doc_type == "structured":
                    self.logger.debug("Found doc type of `structured`")
                    parsed_content = StructuredDocument.from_json(doc_json)
                elif doc_type == "core":
                    self.logger.debug("Found doc type of `core`")
                    parsed_content = CoreDocument.from_json(doc_json)
                else:
                    raise Exception(f"Unknown doc type [{doc_type}] for API v2 Index, you must provide a type attribute (`core` or `structured`).")
            else:
                parsed_content = content
        else:
            raise Exception("You must specify either `doc_path`, `doc_json` or `content` parameters.")

        self.logger.info(f"Creating new document with id [{parsed_content.id}]")
        self.index_api.create_corpus_document(corpus_key, CreateDocumentRequest(parsed_content))